#include "PulseAudioCPP.h"
#include <iostream>
#include "pulseaudioEnumPrint.h"
#include <fstream>

PulseAudioCPP::PulseAudioCPP(std::string appName)
	:mp_context(NULL)
	, mp_mainLoop(NULL)
	, mp_threadedLoop(NULL)
	,mp_buffer(new char[4096])
{

	mp_threadedLoop = pa_threaded_mainloop_new();
	mp_mainLoop = pa_threaded_mainloop_get_api(mp_threadedLoop);
	mp_context = pa_context_new(mp_mainLoop, "VBAN Audio Server");

	pa_context_set_state_callback(mp_context, PulseAudioCPP::contextReadyCB, this);



	pa_threaded_mainloop_lock(mp_threadedLoop);
	pa_threaded_mainloop_start(mp_threadedLoop);

	pa_context_connect(mp_context, NULL, PA_CONTEXT_NOAUTOSPAWN, NULL);

	while (1)
	{
		pa_context_state_t context_state = pa_context_get_state(mp_context);
		if (context_state == PA_CONTEXT_READY) break;
		pa_threaded_mainloop_wait(mp_threadedLoop);
	}

	pa_context_subscribe(mp_context, PA_SUBSCRIPTION_MASK_ALL, PulseAudioCPP::contextCB, this);
	pa_context_set_subscribe_callback(mp_context, PulseAudioCPP::subscriptionCB, this);


}

PulseAudioCPP::~PulseAudioCPP()
{
	if (mp_context)
		pa_context_disconnect(mp_context);
}

void PulseAudioCPP::createStream(std::string name)
{
	pa_sample_spec spec;
	spec.channels = 2;
	spec.format = pa_sample_format_t::PA_SAMPLE_FLOAT32LE;
	spec.rate = 44100;

	pa_channel_map map;
	map.channels = 2;
	map.map[0] = PA_CHANNEL_POSITION_FRONT_LEFT;
	map.map[1] = PA_CHANNEL_POSITION_FRONT_RIGHT;
	 
	pa_stream* newStream = pa_stream_new(mp_context, name.c_str(), &spec, &map);

	pa_buffer_attr buff;
	buff.fragsize = 1400;	//VBAN Packet size
	buff.maxlength = (uint32_t)-1;
	buff.minreq = (uint32_t)-1;
	buff.prebuf = (uint32_t)-1;
	buff.tlength = (uint32_t)-1;

	pa_stream_set_state_callback(newStream, PulseAudioCPP::streamNotifyCB, this);
	pa_stream_set_read_callback(newStream, PulseAudioCPP::streamRequestCB, this);

	pa_stream_connect_record(newStream, NULL, &buff, pa_stream_flags_t::PA_STREAM_NOFLAGS);

	// Wait for the stream to be ready
	for (;;) {
		pa_stream_state_t stream_state = pa_stream_get_state(newStream);
		if (stream_state == PA_STREAM_READY) break;
		pa_threaded_mainloop_wait(mp_threadedLoop);
	}

	pa_threaded_mainloop_unlock(mp_threadedLoop);

	mp_streamList.push_back(newStream);
}

void PulseAudioCPP::subscriptionCB(pa_context* c, pa_subscription_event_type_t t, uint32_t idx, void* instance)
{
	auto ins = (PulseAudioCPP*)instance;

	auto t1 = pa_subscription_event_type_t(t & 0x0f);
	auto t2 = pa_subscription_event_type_t(t & 0xf0);
	std::cout << "Subscription callback: " << idx << " " << t1 << " " << t2 << std::endl;


	if (t2 == pa_subscription_event_type_t::PA_SUBSCRIPTION_EVENT_NEW && t1 == pa_subscription_event_type_t::PA_SUBSCRIPTION_EVENT_SINK_INPUT)
	{

	}
}

void PulseAudioCPP::contextCB(pa_context* c, int success, void* instance)
{
	auto ins = (PulseAudioCPP*)instance;

	std::cout << "Success: " << success << std::endl;
}

void PulseAudioCPP::contextReadyCB(pa_context* context, void* instance)
{
	auto ins = (PulseAudioCPP*)instance;

	std::cout << "Context state callback!" << std::endl;
	pa_threaded_mainloop_signal((pa_threaded_mainloop*)ins->mp_threadedLoop, 0);
}

void PulseAudioCPP::readStream()
{
	char* bufferp;
	size_t size = 0;
	auto s = mp_streamList.front();
	if (s == NULL)
		return;
	pa_stream_peek(s, (const void**)&bufferp, &size);

	m_bytesInBuffer = size;
	for (int i = 0; i < size; i++)
		mp_buffer[i] = bufferp[i];

	//std::cout << "Samples: " << size / 8 << std::endl;
	this->notify(this);
	
	pa_stream_drop(s);
}

void PulseAudioCPP::streamSuccessCB(pa_stream* s, int success, void* userdata)
{
}

void PulseAudioCPP::streamRequestCB(pa_stream* p, size_t nbytes, void* userdata)
{
	auto instance = (PulseAudioCPP*)userdata;
	instance->readStream();
}

void PulseAudioCPP::streamNotifyCB(pa_stream* p, void* userdata)
{
	auto ins = (PulseAudioCPP*)userdata;

	std::cout << "Stream Notify callback!" << std::endl;
	pa_threaded_mainloop_signal((pa_threaded_mainloop*)ins->mp_threadedLoop, 0);
}

void PulseAudioCPP::streamEventCB(pa_stream* p, const char* name, pa_proplist* pl, void* userdata)
{
}
